/*
    menu - img
*/
import $ from '../../util/dom-core.js'
import imgUtl from '../../util/imageUtil.js'
import { getRandom, getParentByClassname, arrForEach, uploadToAliyun } from '../../util/util.js'
import Panel from '../panel.js'

// 构造函数
function Image(editor) {
    this.editor = editor
    const imgMenuId = getRandom('lsiten-e-img')
    this.$elem = $('<div class="lsiten-e-menu" id="' + imgMenuId + '"><i class="lsiten-e-icon-image"></i></div>')
    editor.imgMenuId = imgMenuId
    this.type = 'panel'

    // 当前是否 active 状态
    this._active = false
}

// 原型
Image.prototype = {
    constructor: Image,

    onClick: function () {
        this.$border = null;                
        let $selectionElem = this.editor.page.editorPosition.currentLine;
        let borderText = getParentByClassname($selectionElem, 'js-lsiten-border');
        if (borderText) { 
          let size = borderText.getSizeData();
          this.$border = borderText;
          this._createFileForm(size);
          this.$input[0].value = '';
          this.$input[0].click();
        } else {
          this.$border = null;          
          this.editor.message.showMessage('请在框内进行图片编辑');
        }
    },
    // 创建上传Form
    _createFileForm: function (size) {
      if (this.$form) {
        return '';
      }
      let $form = $('<form class="lsiten-image-file-upload"></form>');
      $form.css('width', 0);
      $form.css('height', 0);
      let $input = $('<input type = "file" accept="image/gif, image/png, image/jpg, image/jpeg"/>');
      $form.append($input);
      this.$form = $form;
      this.$input = $input;
      this.$input.on('change', (e) => {
        this._fileUpload(size);
      })
      $(document.body).append($form);
    },
    _fileUpload: function(size) {
      let file = this.$input[0].files[0];
      let message = this.editor.message;
      const editor = this.editor;
      let policyUrl = editor.config.policyUrl;
      uploadToAliyun(file, policyUrl).then(data => {
         if (data.result === '00') {
           let src = data.data.url;
           let srcStore = data.data.urlStore;
           this._doCmd({
             src: src,
             store: srcStore,
             bsize: size
           });         
         } else {
            message.showMessage(data.msg || '上传出错，请重试！');
         }
      })
    },
    _doCmd: function (img) {
      let imageTool = new imgUtl(img.src);
      let _this = this;
      imageTool.getSize(complete);
      function complete() {
        let owidth = imageTool.owidth;
        let bsize = img.bsize;
        let bwidth = bsize.width - 30;
        if (owidth >= bwidth) {
          let ratio = bwidth / owidth;
          imageTool.resizeByRatio(ratio);
        }
        let oheight = imageTool.size.h;
        let bheight = bsize.height - 22;
        if (oheight > bheight) {
          let ratio = bheight / oheight;
          imageTool.resizeByRatio(ratio);
        }
        let html = $('<img class="lsiten-js-image-class" style="width: ' + imageTool.size.w + 'px; height: ' + imageTool.size.h + 'px" src="' + img.src + '" data-store = "' +img.store+ '"/>');
        html.css('position', 'absolute').css('top', '0').css('left', '0');
        _this.$border.append(html);      
      }
    },
    // 试图改变 active 状态
    tryChangeActive: function (e) {
        const editor = this.editor
        const $elem = this.$elem
        if (editor._selectedImg) {
            this._active = true
            $elem.addClass('lsiten-e-active')
        } else {
            this._active = false
            $elem.removeClass('lsiten-e-active')
        }
    }
}

export default Image