import $ from '../util/dom-core.js';
import { getParentByClassname, isEmptyElement } from '../util/util.js';
import { gennerOneLines } from '../util/lineUtil.js';
import uuid from 'uuid';

/**
 * 
 * @param {DomElement} $border 需要添加题的框
 * @param {JSON} data  题目数据
 * @param {Objec} editor 编辑器对象
 */
let generateWriting = function ($border, data, editor) {
  let type = parseInt($border.attr('data-type'));
  if (type === 1) {
    editor.message.showMessage('请在主观题框中添加作文题！');
    return '';
  }
  $border.css('padding', '10px 0');
  let pnum = data.pnum;
  let total = parseInt(data.nums) || 1200;
  let $box = $('<div class="lsiten-question-writing js-lsiten-question"></div>');
  $box.attr('data-type', 4);
  $box.attr('data-pnum', pnum);
  $box.attr('data-id', uuid());
  $box.attr('data-total', total);
  // 1、生成题号区
  let $title = $('<div class="lsiten-title"></div>');
  $title.css('padding', '0 15px');
  // 在主观题内添加
  let $currentLine = editor.page.editorPosition.currentLine;
  if (!$currentLine) {
    console.log("%c请在框内添加该题！", "color:red");
    return '';
  }
  let $borderContent = getParentByClassname($currentLine, 'border-content');
  if (!$borderContent) {
    console.log("%c请在框内添加该题！", "color:red");
    return '';
  }
  let lineHeight = $borderContent.parent()[0].getAttribute('data-blineheight');
  gennerOneLines(1, $title, pnum, lineHeight);
  $box.append($title);
  // 2、生成格子区域
  let column = getColumnCount(editor);
  let $grid = generateGrid(column, total);
  $box.append($grid);
  
  let questionBox = getParentByClassname($currentLine, 'js-lsiten-question');
  if (questionBox) {
    $currentLine = parseInt(questionBox.attr('data-type')) > 2 ? questionBox : questionBox.parent();
  }
  $box.insertAfter($currentLine);
  if ( isEmptyElement($borderContent[0].firstChild) ) {
    $borderContent[0].removeChild($borderContent[0].firstChild);
  }
  $title[0].firstChild.focus();
  editor.page._updatePositionInfo(null, $($title[0].firstChild));
}

/**
 * 
 * @param {Object} editor 编辑器
 */
function getColumnCount (editor) {
  let pageSize = editor.pageSize;
  let column = parseInt(editor.columnNumber);
  let maxColumn = 19;
  if (pageSize === 'A4' && column === 1) {
    maxColumn = 17;
  }
  if (pageSize === 'A3' && column === 1) {
    maxColumn = 38;
  }
  if (pageSize === 'A3' && column === 2) {
    maxColumn = 19;
  }
  if (pageSize === 'A3' && column === 3) {
    maxColumn =9;
  }

  return maxColumn;
}


/**
 * function 生成作文格子
 * @param {int} column 行格子数
 * @param {int} total 格子总数
 */
function generateGrid (column, total) {
  let $borderBox = $('<div class="lsiten-border-box" contenteditable="false"></div>');
  let rows = Math.ceil(total / column);
  let columnIndex = 0;
  let width = column * 34 + 'px';
  let lastColumn = column - 1;
  for (let i = 0; i < rows; i++) {
    let $row = $('<div class="js-lsiten-writting-row"></div>');
    $row.attr('data-row-index', i);
    $row.css('height', '32px')
        .css('width', width)
        .css('line-height', '32px')
        .css('white-space', 'nowrap')
        .css('text-align', 'left')
        .css('margin-top', '8px')
        .css('margin-left', 'auto')
        .css('margin-right', 'auto');
    for (let j = 0; j < column; j++) {
      let $column = $('<div class="js-lsiten-writting-column"></div>');
      $column.attr('data-column-index', ++columnIndex);   
      
      $column.css('border-left', '1px solid #000')
              .css('border-top', '1px solid #000')
              .css('border-bottom', '1px solid #000')
              .css('width', '34px')
              .css('height', '34px')
              .css('position', 'relative')
              .css('box-sizing', 'border-box')
              .css('display', 'inline-block')
              .css('margin', '0');
      j === lastColumn && $column.css('border-right', '1px solid #000');
      if (columnIndex % 400 === 0) {
        let $tips = $('<div class="lsiten-writting-border-tips">' + columnIndex +'字</div>');
        $tips.css('font-size', '12px')
             .css('position', 'absolute')
             .css('transform', 'scale(0.5)')
             .css('font-size', '8px')
             .css('left', '0px')
             .css('bottom', '-20px');
        $column.append($tips);
      }
      $row.append($column);
    }
    $borderBox.append($row);
  }

  return $borderBox;
}
export default {
  generateWriting: generateWriting
}
