import $ from './dom-core';
import {isEmptyElement} from './util.js'
import {gennerOneLine} from './lineUtil.js'
let questionUtils = function () {

}

questionUtils.prototype = {
  constructor: questionUtils,
  /**
   * function 切分题目框
   * @param {DomElement} $border 题框
   * @param {DomElement} $column 栏dom
   */
  _splitBorder: function ($border, $column) {
    let type = parseInt($border.attr('data-type'));
    switch (type) {
      case 1:
      return this._spiltObjectBorder($border,  $column);
      break;
      case 2:
      return this._spiltSubjectBorder($border,  $column);      
      break;
    }
  },
  /**
   * function 切分客观题框
   * @param {DomElement} $border 题框
   */
  _spiltObjectBorder: function ($border,  $column) {
    let $nextBorder = $border.clone(false);
    let childs = $border.childNodes();
    let cLength = childs.length;
    let i = cLength - 1;
    while (i >= 0) {
      if (childs[i]) {
        if (childs[i].nodeType === 3) {
          $border[0].removeChild(childs[i]);
        } else {
          let $child = $(childs[i]);
          let classname = $child.attr('class');
          if (classname && classname.indexOf('js-options-row') > -1) {
            let $nextRow = $child.clone(false);
            this._splitObjectRow($nextRow, $child, $border, $column);
            if (!isEmptyElement($nextRow[0])) {
              this.insertDom($nextBorder, $nextRow);
            }
          } else {
            $child.remove();
          }
        }
      }
      i--;
    }

    if (!isEmptyElement($nextBorder[0])) {
      return $nextBorder;
    } else {
      return false;
    }
  },
  /**
   * function 处理客观题的row
   * @param {DomElement} $nextDiv 下一段的div
   * @param {DomElement} $item 文本节点
   * @param {DomElement} $border 所在框
   * @param {DomElement} $bigColumn 栏dom
   */
  _splitObjectRow: function ($nextDiv, $item, $border, $bigColumn) {
    let borderSize =  $border.getSizeData();
    let csize = $bigColumn.getSizeData();
    if (borderSize.bottom >= csize.bottom) {
      let i = 0;
      let firstColumn = $item[0].firstChild;
      if (firstColumn) {
        let lastKey = $(firstColumn.firstChild).childNodes().length - 1;
        if (lastKey > -1) {
          $item.childNodes().forEach(column => {
            this._insertLastRow($nextDiv, column, i++, lastKey);
          });
          if (!this._isEmptyRow($item)) {
            // 更新栏高
            let cententHeight = 0;
            $item.find('.js-option-column').forEach(column => {
              let contentSize = $(column.firstChild).getSizeData();
              cententHeight = contentSize.height;
              $(column).css('min-height', cententHeight + 'px');
            });
            this._splitObjectRow($nextDiv, $item, $border,  $bigColumn);
          } else {
            $item.remove();
          }
          
        }
      }
    }
  },
  /**
   * function 插入该row的最后一行
   */
  _insertLastRow: function ($nextRow, column, key, lastKey) {
    let nextColumns = $nextRow.childNodes();
    let questions = $(column.firstChild).find('.js-lsiten-question');
    let $nextColumnContent = null;
    if (nextColumns[key]) {
      $nextColumnContent = $(nextColumns[key].firstChild);
    } else {
      let $nextColumn = $(column).clone(false);
      $nextColumnContent = $(column.firstChild).clone(false);
      $nextColumn.append($nextColumnContent);
      $nextRow.append($nextColumn);
    }
    if (questions[lastKey]) {
      this.insertDom($nextColumnContent, $(questions[lastKey]));
    }
  },
  /**
   * function 判断行是否为空
   */
  _isEmptyRow: function ($row) {
    let status = true;
    $row.childNodes().forEach(column => {
      if (!isEmptyElement(column)) {
        status = false;
      }
    })
    return status;
  },
  /**
   * function 插入dom
   * @param {DomElement}  $nextDom 下一栏暂存
   * @param {DomElement}  $dom 需要插入的dom
   */
  insertDom($nextDom, $dom) {
    let firstChild = $nextDom[0].firstChild;
    if (firstChild) {
      $dom.insertBefore($(firstChild));
    } else {
      $nextDom.append($dom);
    }
  },
    /**
   * function 切分主观题框
   * @param {DomElement} $border 题框
   * @param {Object} $bigColumn 栏size
   */
  _spiltSubjectBorder: function ($border, $bigColumn) {
    let $nextBorder = $border.clone(false);
    let $content = $($border[0].firstChild);
    if ($content.attr('class') === 'border-content') {
      let childrens = $content.childNodes();
      let $nextContent = $content.clone(false);
      let index = childrens.length - 1;
      while (index >= 0 ) {
        let line = childrens[index];
        if (line && line.className) {
          if (line.className.indexOf('js-lsiten-line') > -1) {
            // 普通一行或者是单选题可判断题
            let $line = $(line);
            let borderSize = $border.getSizeData();
            let csize = $bigColumn.getSizeData();
            if (borderSize.bottom >= csize.bottom) {
              this.insertDom($nextContent, $line);
            }
          } else if (line.className.indexOf('js-lsiten-options-box') > -1) {
            // 选作题组
            let $line = $(line);
            let borderSize = $border.getSizeData();
            let csize = $bigColumn.getSizeData();
            if (borderSize.bottom >= csize.bottom) {
              this.insertDom($nextContent, $line);
            }
          } else if (line.className.indexOf('js-lsiten-question') > -1) {
            let $line = $(line);
            let questionType = parseInt($line.attr('data-type'));
            switch (questionType) {
              case 1:
              this._handleChoice($line, $nextContent, $bigColumn);
              break;
              case 2:
              this._handleJudge($line, $nextContent, $bigColumn);              
              break;
              case 3:
              this._handleSubject($line, $nextContent, $border, $bigColumn);              
              break;
              case 4:
              this._handleWritting($line, $nextContent, $border, $bigColumn);              
              break;
            }
          }
        } else {
          line && $content[0].removeChild(line);
        }
        index--;
      }
      
      if (isEmptyElement($content[0])) {
        $border.remove();
      }
      if ($nextContent.childNodes().length > 0) {
        $nextBorder.append($nextContent);
        $border.addClass('js-split-border');
        $nextBorder.addClass('js-split-border');
        return $nextBorder;
      } else {
        return false;
      }

    } else {
      $border.remove();
      return false;
    }
  },
  /**
   * function 处理选择题div
   * @param {DomElement} $item 纯普通div
   * @param {DomElement} $nextContent 下一个框暂存
   * @param {DomElement} $bigColumn 栏dom
   */
  _handleChoice: function ($item, $nextContent, $bigColumn) {
    let lineSize = $item.getSizeData();
    let csize = $bigColumn.getSizeData();
    if (lineSize.top >= csize.bottom) {
      let lineHeight = $nextContent.parent()[0].getAttribute('data-blineheight');      
      let newLine = gennerOneLine(false, lineHeight);
      newLine.append($item);
      $nextContent.append(newLine);
    }
  },
  /**
   * function 处理判断题div
   * @param {DomElement} $item 纯普通div
   * @param {DomElement} $nextContent 下一个框暂存
   * @param {DomElement} $bigColumn 栏size
   */
  _handleJudge: function ($item, $nextContent, $bigColumn) {
    let lineSize = $item.getSizeData();
    let csize = $bigColumn.getSizeData();
    if (lineSize.top >= csize.bottom) {
      let lineHeight = $nextContent.parent()[0].getAttribute('data-blineheight');            
      let newLine = gennerOneLine(false, lineHeight);
      newLine.append($item);
      $nextContent.append(newLine);
    }
  },
  /**
   * function 处理客观题Div
   * @param {DomElement} $item 纯普通div
   * @param {DomElement} $nextContent 下个框暂存
   * @param {DomElement} $border 所在框
   * @param {DomElement} $bigColumn 栏dom
   */
  _handleSubject: function ($item, $nextContent, $border, $bigColumn) {
    let $nextSubject = $item.clone(false);
    let childs = $item.childNodes();
    let index = childs.length - 1;
    while (index >= 0 ) {
      let line = childs[index];
      if (line && line.className) {
        if (line.className.indexOf('js-lsiten-line') > -1) {
          // 普通一行
          let $line = $(line);
          let bSize = $border.getSizeData();
          let csize = $bigColumn.getSizeData();
          if (bSize.bottom >= csize.bottom) {
            this.insertDom($nextSubject, $line);
          }
        } else {
          $(line).remove();
        }
      } else {
        line && $paragraph[0].removeChild(line);
      }
      index--;
    }

    if (isEmptyElement($item[0])) {
      $item.remove();
    }

    if ($nextSubject.childNodes().length > 0) {
      this.insertDom($nextContent, $nextSubject);
      $nextSubject.addClass('js-split-question');
      $item.addClass('js-split-question');
    }
  },
  /**
   * function 处理作文题Div
   * @param {DomElement} $item 纯普通div
   * @param {DomElement} $nextContent 下个框暂存
   * @param {DomElement} $border 所在框
   * @param {DomElement} $bigColumn 栏dom
   */
  _handleWritting: function ($item, $nextContent, $border, $bigColumn) {
    let childs = $item.childNodes();
    let cLength = childs.length;
    let csize = $bigColumn.getSizeData();
    let $nextWritting = $item.clone(false);
    if (cLength > 0) {
      let i = cLength - 1;
      while (i >= 0) {
        let bSize = $border.getSizeData();
        csize = $bigColumn.getSizeData();
        let child = childs[i];
        let $child = $(child);
        if (bSize.bottom >= csize.bottom) {
          if (child.className.indexOf('lsiten-title') > -1) {
            if (!isEmptyElement(child)) { 
              // 作文标题
              let $nextLsitenTitle = $child.clone(false);
              let titleChilds = $child.childNodes();
              let titleIndex = titleChilds.length - 1;
              while(titleIndex >= 0) {
                let line = titleChilds[titleIndex];
                if (line && line.className) {
                  if (line.className.indexOf('js-lsiten-line') > -1) {
                    // 普通一行
                    let $line = $(line);
                    bSize = $border.getSizeData();
                    csize = $bigColumn.getSizeData();
                    if (bSize.bottom >= csize.bottom) {
                      this.insertDom($nextLsitenTitle, $line);
                    }
                  } else {
                    $(line).remove();
                  }
                } else {
                  line && $paragraph[0].removeChild(line);
                }
                titleIndex--;
              }
              if ($nextLsitenTitle.childNodes().length > 0) {
                this.insertDom($nextWritting, $nextLsitenTitle);
              }
            } else {
              $child.remove();
            }
          } else if (child.className.indexOf('lsiten-border-box') > -1){
            // 作文格子
            let $nextLsitenBorder = $child.clone(false);
            let rows = $child.childNodes();
            let rowLength = rows.length;
            if ( rowLength > 0 ) {
              let k = rowLength - 1;
              while ( k >= 0 ) {
                let bSizeBorder = $border.getSizeData();
                csize = $bigColumn.getSizeData();
                if (bSizeBorder.bottom >= csize.bottom) {
                  let $rowItem = $(rows[k]);
                  this.insertDom($nextLsitenBorder, $rowItem);
                }
                k--;
              }
            } else {
              $nextLsitenBorder.remove();
            }

            if ($nextLsitenBorder.childNodes().length > 0) {
              this.insertDom($nextWritting, $nextLsitenBorder);
              $nextLsitenBorder.addClass('js-split-writting-border');
            } 
          } else {
            $child.remove();
          }
        }
        i--;
        isEmptyElement($child[0]) && $child.remove();
      }
    }

    if ($nextWritting.childNodes().length > 0) {
      this.insertDom($nextContent, $nextWritting);
      $nextWritting.addClass('js-split-question');
      $item.addClass('js-split-question');
    }

    if (isEmptyElement($item[0])) {
      $item.remove();
    }
  }
}

export default questionUtils;
