/*
    工具
*/
import Ajax from './ajax'
// 和 UA 相关的属性
export const UA = {
    _ua: navigator.userAgent,

    // 是否 webkit
    isWebkit: function () {
        const reg = /webkit/i
        return reg.test(this._ua)
    },

    // 是否 IE
    isIE: function () {
        return 'ActiveXObject' in window
    }
}

// 遍历对象
export function objForEach(obj, fn) {
    let key, result
    for (key in obj) {
        if (obj.hasOwnProperty(key)) {
            result = fn.call(obj, key, obj[key])
            if (result === false) {
                break
            }
        }
    }
}

// 遍历类数组
export function arrForEach(fakeArr, fn) {
    let i, item, result
    const length = fakeArr.length || 0
    for (i = 0; i < length; i++) {
        item = fakeArr[i]
        result = fn.call(fakeArr, item, i)
        if (result === false) {
            break
        }
    }
}

// 获取随机数
export function getRandom(prefix) {
    return prefix + Math.random().toString().slice(2)
}

// 替换 html 特殊字符
export function replaceHtmlSymbol(html) {
    if (html == null) {
        return ''
    }
    return html.replace(/</gm, '&lt;')
                .replace(/>/gm, '&gt;')
                .replace(/"/gm, '&quot;')
                .replace(/(\r\n|\r|\n)/g, '<br/>')
}

// 返回百分比的格式
export function percentFormat(number) {
    number = (parseInt(number * 100))
    return number + '%'
}

// 判断是不是 function
export function isFunction(fn) {
    return typeof fn === 'function'
}

export function getParentNodeByClass (enode, className) {
  if (!enode) {
    return null;
  }
  if (enode.nodeType === 1 && enode.tagName.toLowerCase() === 'body') {
    return null;
  }
  let nodeClass = enode.className;
  if (nodeClass && nodeClass.indexOf(className) > -1) {
    return enode
  } else {
    return getParentNodeByClass(enode.parentNode, className)
  }
}
export function getParentByClassname($el, classname) {
    if (!$el || !$el.length) {
      return null;
    }
    if ($el.getNodeName().toLowerCase() === 'body' ) {
      return null;
    }
    let eclassname = $el.attr('class')
    if (eclassname && eclassname.indexOf(classname) > -1) {
      return $el
    } else {
      return getParentByClassname($el.parent(), classname)
    }
  }

  export function trim(str) {
    str = str.replace(/[ | ]*\n/g,'\n'); //去除行尾空白
    str=str.replace(/ /ig,'');//去掉 
    str=str.replace(/(^\s*)|(\s*$)/g,'');//去掉 
    return str;
  }

  export function debounce (func,wait,immediate = false) {
    // immediate 默认为false
    let timeout = null; let args = null; let context = null; let timestamp; let result = null;
  
    let later = () => {
      let last = new Date().getTime() - timestamp;
      if( last < wait && last > 0 ){
        timeout = setTimeout(later, wait - last);
      }else{
        timeout = null;
        if (!immediate) {
          result = func.apply(context, args);
          if (!timeout) {
            context = args = null;
          }
        }
      }
    };
  
    return function() {
      context = this;
      args = arguments;
      timestamp = new Date().getTime();
      // 第一次调用该方法时，且immediate为true，则调用func函数
      let callNow = immediate && !timeout;
      // 在wait指定的时间间隔内首次调用该方法，则启动计时器定时调用func函数
      if (!timeout) timeout = setTimeout(later, wait);
      if (callNow) {
        result = func.apply(context, args);
        context = args = null;
      }
      return result;
    };
  
  }
  
  export function throttle (func, wait, options) {
    /* options的默认值
       *  表示首次调用返回值方法时，会马上调用func；否则仅会记录当前时刻，当第二次调用的时间间隔超过wait时，才调用func。
       *  options.leading = true;
       * 表示当调用方法时，未到达wait指定的时间间隔，则启动计时器延迟调用func函数，若后续在既未达到wait指定的时间间隔和func函数又未被调用的情况下调用返回值方法，则被调用请求将被丢弃。
       *  options.trailing = true;
       * 注意：当options.trailing = false时，效果与上面的简单实现效果相同
       */
    let context, args, result;
    let timeout = null;
    let previous = 0;
    if (!options) {
      options = {};
    }
    let later = function() {
      previous = options.leading === false ? 0 : new Date().getTime();
      timeout = null;
      result = func.apply(context, args);
      if (!timeout) {
        context = args = null;
      }
    };
    return function() {
      let now = new Date().getTime();
      if (!previous && options.leading === false) {
        previous = now;
      }
      // 计算剩余时间
      let remaining = wait - (now - previous);
      context = this;
      args = arguments;
      // 当到达wait指定的时间间隔，则调用func函数
      // 精彩之处：按理来说remaining <= 0已经足够证明已经到达wait的时间间隔，但这里还考虑到假如客户端修改了系统时间则马上执行func函数。
      if (remaining <= 0 || remaining > wait) {
        // 由于setTimeout存在最小时间精度问题，因此会存在到达wait的时间间隔，但之前设置的setTimeout操作还没被执行，因此为保险起见，这里先清理setTimeout操作
        if (timeout) {
          clearTimeout(timeout);
          timeout = null;
        }
        previous = now;
        result = func.apply(context, args);
        if (!timeout) context = args = null;
      } else if (!timeout && options.trailing !== false) {
        // options.trailing=true时，延时执行func函数
        timeout = setTimeout(later, remaining);
      }
      return result;
    };
  }

export function throttleSimple(fn, delay){
 	let timer = null;
 	return function(){
 		let context = this, args = arguments;
 		clearTimeout(timer);
 		timer = setTimeout(function(){
 			fn.apply(context, args);
 		}, delay);
 	};
 };
 
 export function uploadToAliyun (file, policyUrl) {
  return new Promise((resolve, reject) => {
    let Api = new Ajax();
    // 1、获取Policy
    Api.post(policyUrl, 'inType=41&inFiletype=' + (file.name.substring((file.name.lastIndexOf(".") + 1))) ).then(data => {
      let responce = data;
      if (responce.result === '00') {
        let resData = responce.data;
        let formdata = new  window.FormData();
        formdata.append('key', resData.dir + resData.filename + (file.name.substring((file.name.lastIndexOf(".")))));
        formdata.append('policy', resData.policy);
        formdata.append('OSSAccessKeyId', resData.accessid);
        formdata.append('success_action_status', 200);
        formdata.append('signature', resData.signature);
        formdata.append('file', file);
        Api.post(resData.host, formdata, 'formdata').then(data => {
          let returndata = {
            result: '00',
            data: {
              url: decodeURI(resData.url),
              urlStore: resData.urlStore
            }
          }
          resolve(returndata);
        }).catch(err => {
          reject(err);
        })
      } else {
        reject('policy获取失败！')
      }
    }).catch(err => {
      reject(err);
    })
  })
}

/**
 * function 判断是否为空的元素
 * @param {Node}  node
 */
export function isEmptyElement (enode) {
  if (enode.nodeType === 3) {
    return !enode.nodeValue.length;
  } else {
    if (!enode.tagName || enode.tagName.toLowerCase() === 'br') {
      return true;
    } else if (!enode.childNodes || enode.childNodes.length < 1) {
      return true;
    } else {
      let html = enode.innerHTML.toLowerCase();
      if (html.length < 1 || html === '<br>' || html === '<br/>' || html === '') {
        return true;
      } else {
        return false;
      }
    }
  }
}
