import $ from './util/dom-core.js';
import { gennerOneLine } from './util/lineUtil.js';
import { getPasteText, getPasteHtml, getPasteImgs } from './util/paste-handle.js';
import imgUtl from './util/imageUtil.js'
import Jquery from 'jquery';
import yzPage from './yzPage.js';
import { UA, arrForEach, isFunction, replaceHtmlSymbol, getParentByClassname, trim, throttle, isEmptyElement, getParentNodeByClass, uploadToAliyun } from './util/util.js';

let yzPageManager = function (editor) {
  this.editor = editor;
  this.id = ('lsiten_editor_page_'+Math.random()).replace('.','_');
  this.pageIndex = 0;
  this.currentPage = null;
  this.pages = [];
  // enter键状态
  this._event = null;
  this._checkPageOutStatus = false;  
  this._checkPageBackStatus = false;  

  // 输入法监听 
  this.inputType = false;

  this.$el = $('<div id = " ' + this.id + ' " class="lsiten-pages-view"></div>');

  this.ready = false;
  this.editorPosition = {};
  this._$handleImg = null;
  this._init();
}

yzPageManager.prototype = {
  constructor: yzPageManager,
  _init: function () {
    this._initStyles();
    if (!this.editor.cardHtml ||  this.editor.cardHtml.length < 1) {
      this._addPage();
    }
    this._initEvents();

    this._initDrageDiv();
  },
  /**
   * function 初始化编辑后的数据
   */
  parseCardHtml: function (cardHtml) {
    const editor = this.editor;
    this._initStatus = true;
    this._removePages();
    cardHtml.forEach(pageHtml => {
      let $pageHtml = $(pageHtml);
      this._addPage($pageHtml);
    });
    this.pages.forEach(page => {
      page.$colums.forEach($colum => {
        page._updateCrossColumn($colum);
      })
    })
  },
  /**
   * function 删除所有页面
   */
  _removePages: function () {
    this.currentPage = null;
    this.pages = [];
    this.$el.html('');
    this.pageIndex = 0;
  },
  getPagePlaceholderTemlpate: function (pageIndex) {
    // 添加分页显示效果
    let $breakDiv = $('<div class="page-break" data-page-id="' + (pageIndex) + '"></div>');
    let $topSpan = $('<div class="spanMark topSpan">&#8203;</div>');
    $topSpan.css('display', 'block').css('height', '71.5px');
    // 分页符
    let $middleSpan = $('<div class="spanMark middleSpan lsiten-page-break-span">&#8203;</div>');
    $middleSpan.css('page-break-before', 'always');
    $middleSpan.css('display', 'block').css('height', '0');  
    let $bottomSpan = $('<div class="spanMark bottomSpan">&#8203;</div>');
    $bottomSpan.css('height', '71.5px').css('display', 'block');

    // 添加页眉页脚   
    $breakDiv.append($topSpan);
    $breakDiv.append($middleSpan);
    $breakDiv.append($bottomSpan);
    $breakDiv.css('height', '153px');
    return $breakDiv;
  },
  /**
   * function 初始化页面的样式
   */
  _initStyles: function () {
    this.$el.css('width', this._pagesWidth[this.editor.pageSize])
            .css('background', '#ffffff')
            .css('position', 'relative')
            .css('padding', '70px!important')
            .css('box-sizing', 'border-box')
            .css('min-width', '0')
            .css('box-shadow', '0 2px 7px rgba(0,0,0,.1)')
            .css('overflow', 'hidden')
            .css('margin', '0 auto')
            .css('height', this._pagesHeight[this.editor.pageSize]);
  },
  /**
   * function 添加一页
   */
  _addPage: function ($page) {
    this._addBreakPlaceholder();    
    let page = new yzPage(this,  $page || null);
    this.pages.push(page);
    this.pageIndex++;
    if (!$page) {
      page.addHeader();
    }
    return page;
  },
  /**
   * function 通过key值获取下一页
   * @param {number} key page的key值
   * @param {Boolean} isAddPage 如果没有是否添加一页
   */
  getNextPageByKey: function (key, isAddPage) {
    let page = this.pages[key];
    if (page) {
      return page;
    } else {
      if (isAddPage) {
        return this._addPage();
      } else {
        return false;
      }
    }
  },
  _addBreakPlaceholder: function () {
    if (this.pageIndex < 1) {
      return '';
    }
    // 添加分页显示效果
    let $breakDiv = $('<div class="page-break js-page-break-' + (this.pageIndex) + '" data-page-id="' + (this.pageIndex) + '"></div>');
    let $topSpan = $('<div class="spanMark topSpan">&#8203;</div>');
    $topSpan.css('display', 'block').css('height', '71.5px');
    // 分页符
    let $middleSpan = $('<div class="spanMark middleSpan lsiten-page-break-span">&#8203;</div>');
    $middleSpan.css('page-break-before', 'always');
    $middleSpan.css('display', 'block').css('height', '10px').css('background-color', '#f6f6f6').css('box-shadow', 'inset 0 7px 7px -5px rgba(0, 0, 0, 0.1), inset 0 -5px 5px -7px rgba(0, 0, 0, 0.1), 7px 0 7px #f6f6f6, -7px 0 7px #f6f6f6');  
    let $bottomSpan = $('<div class="spanMark bottomSpan">&#8203;</div>');
    $bottomSpan.css('height', '71.5px').css('display', 'block');

    // 添加页眉页脚   
    $breakDiv.append($topSpan);
    $breakDiv.append($middleSpan);
    $breakDiv.append($bottomSpan);
    $breakDiv.css('height', '153px');
    this.$el.append($breakDiv);

    let height = this.pages.length * this._pagesBreakHeight[this.editor.pageSize] + parseInt(this._pagesHeight[this.editor.pageSize]);
    this.$el.css('height', height + 'px');
  },

  _checkEveryPageOut: function (callback) {
    if (this._checkPageOutStatus) {
      return false;
    }
    this._checkPageOutStatus = true;
    let pages = this.pages;
    pages.forEach(page => {
      page._checkContentOut();
    });
    if (typeof callback === 'function') {
      callback();
    }
    this._checkPageOutStatus = false;
  },

  /**
   * function 检测是否有空页并删除
   */
  _checkEveryPageBack: function (callback) {
    if (this._checkPageBackStatus) {
      return false;
    }
    this._checkPageBackStatus = true;
    let pages = this.pages;
    pages.forEach(page => {
      page._checkContentBack();
    });
    if (typeof callback === 'function') {
      callback();
    }
    this._checkPageBackStatus = false;
  },

  _checkRemovePage() {
    let pagesTemp = this.pages.slice(0);
    let i = pagesTemp.length - 1;
    while(i>=0) {
      let page = pagesTemp[i];
      let currentParagraphs = page.$textElem.find('.js-paragraph-view');
      if (currentParagraphs.length && currentParagraphs.length === 1 && currentParagraphs[0].firstChild.className && currentParagraphs[0].firstChild.className.indexOf('js-answer-header') > -1) {
        this.removePage(page.pageIndex);
      }
      i--;
    }
  },
  /**
   * function 移除某一页
   * @param {int} 页面id
   */
  removePage: function (index) {
    this.$el.find('.js-page-break-' + index).remove();
    this.pages[index].$textElem.remove();
    this.pageIndex--;
    this.pages.splice(index,1);
    let i = 0;
    this.pages.forEach(page => {
      page.$textElem.find('.js-column').attr('data-column-page', i++);
    })
    let height = (this.pages.length - 1) * this._pagesBreakHeight[this.editor.pageSize] + parseInt(this._pagesHeight[this.editor.pageSize]);
    this.$el.css('height', height + 'px');
  },
  /**
   * function 添加事件监听 
   */
  _initEvents: function () {
    this._realtimeSavePosition();
    this._enterEvent();
    this._backEvent();
    this._delEvent();

    this._contentchangeEvent();

    // 框内事件菜单
    this._borderEvent();


    this._writtingHandleEvent();

    this._imgHandle();
    this._purseTextHandle();
    this._scrollEvent();
    this._resizeEvent();
    this._dragEvent();
  },
  /**
   * function 文本框事件处理
   */
  _purseTextHandle: function () {
    const $textElem = this.$el;
    // 为图片增加 selected 样式
    let $delete = $('<div class="js-lsiten-text-delete">删除</div>');
    Jquery($textElem[0]).on('focus', '.js-pure-text', function (e) {
      let ele = e.currentTarget;
      let $text = Jquery(ele);
      if (!$text.find('.js-lsiten-text-delete') || !$text.find('.js-lsiten-text-delete').length) {
        $delete =  $('<div class="js-lsiten-text-delete">删除</div>');
        $delete.css('position', 'absolute').css('display', 'inline-block').css('right', '-10px').css('z-index', '999').css('width', 'auto').css('top', '-25px').css('cursor', 'pointer');
        $text.css('border', '1px dashed #efefef').css('border-radius', '3px');
        $text.append($delete);
        $delete.on('click', function (event) {
          $text.remove();
          Jquery($textElem[0]).trigger('removeContent');
          event.stopPropagation();
        })
      }
    })
    // 去掉 selected 样式
    Jquery($textElem[0]).on('click', e => {
      let ele = e.target;
      let pureText = getParentNodeByClass(ele, 'js-pure-text');
      if (!pureText) {
        $textElem.find('.js-pure-text').css('border', 'none').css('border-radius', '0');
        $delete.remove();
      }
    })
  },
  /**
   * function 框相关事件
   */
  _borderEvent: function () {
    let _this = this;
    const $textElem = this.$el;
    Jquery($textElem[0]).on('contextmenu','.js-lsiten-border', (e) => {
      this._createContextMenu(e);
    })
  },
  /**
   * fucntion 创建菜单
   * @param {event} e 事件对象
   * @param {array} menus 数组对象
   */
  _createContextMenu: function (e, menus) {
    let $target = $(e.target);
    let $border = getParentByClassname($target, 'js-lsiten-border');
    let movex=e.pageX;
    let movey=e.pageY;
    !menus && (menus = []);
    this._createMenu(movex, movey, $border, menus);
    e.preventDefault();
  },
  /**
   * function 创建菜单列表
   * @param {number} x x轴坐标 
   * @param {number} y y轴坐标 
   * @param {DomElement} $border 该框 
   * @param {array} menus 数组对象 
   */
  _createMenu: function (x, y, $border, menus) {
    let $menu = $('<div class="js-lsiten-contentmenus"></div>');
    $menu.css('width', '100px')
         .css('height', 'auto')
         .css('background', '#fff')
         .css('position', 'absolute')
         .css('z-index', '9999')
         .css('outline', 'none')
         .css('top', y + 'px')
         .css('border', '1px solid #e4dddd')
         .css('padding', '0')
         .css('box-shadow', '0 3px 7px rgba(0, 0, 0, 0.1)')
         .css('left', x + 'px');
    $menu.attr('tabIndex', 1);

    menus.forEach(item => {
      let $menuItem = $('<div>' + item.name + '</div>');
      $menuItem.css('cursor', 'pointer')
           .css('text-align', 'center')
           .css('border-bottom', '1px solid #e8e3e3')
           .css('padding', '5px 0');
      $menu.append($menuItem);
      if (item.click && typeof item.click === 'function') {
        item.click($menuItem);
      }
    })
    this._commonMenus($menu, $border);
    $(document.body).append($menu);
    $menu.focus();
    Jquery($menu[0]).on('blur', (e) => {
      $menu && $menu.remove();
    })
  },
  /**
   * function 创建菜单列表
   * @param {DomElement} $menu menu的dom
   * @param {DomElement} $border 框的dom
   */
  _commonMenus: function ($menu, $border) {
    // 添加文本框开始
    const message = this.editor.message;
    const $textElem = this.$el;
    let $addText = $('<div>加文本框</div>');
    $addText.css('cursor', 'pointer')
           .css('text-align', 'center')
           .css('border-bottom', '1px solid #e8e3e3')
           .css('padding', '5px 0');
    $menu.append($addText);
    $addText.on('click', () => {
      this._checkNeedBackStatus = true;
      let $prevBorders = this._findPrevBorders($border);
      let lastBorderIndex = $prevBorders.length - 1;
      let $baiscBorder = lastBorderIndex >= 0 ? $($prevBorders[lastBorderIndex]) : $border;
      let $currentParagraph = getParentByClassname($baiscBorder, 'js-paragraph-view');
      if ($currentParagraph) {
        let index = $currentParagraph.attr('data-index');
        let pageIndex = parseInt(index.split('-')[0]);
        if (pageIndex > -1) {
          let currentPage = this.pages[pageIndex];
          if (currentPage) {
            let $paragraph = $('<div class="js-paragraph-view js-pure-text"></div>');
            $paragraph.append(gennerOneLine());
            $paragraph.css('margin', 0)
                      .css('padding', '0 25px')
                      .css('outline', 'none')
                      .css('position', 'relative')
                      .css('box-sizing', 'border-box')
                      .css('white-space', 'pre-wrap')
                      .css('line-height', '1.75')
                      .css('text-align', 'left');
            let index = pageIndex + '-' + currentPage.paragraphIndex++;
            $paragraph.attr('data-index', index);
            $paragraph.insertBefore($currentParagraph);
            setTimeout(() => {
              $paragraph.focus();
            }, 100)
          }
        } else {
          console.log('error');
        }
      } else {
        console.log('error')
      }

      $menu[0].blur();
    })
    // 添加文本框结束

    // 更改框行高开始
    let $addLineHeight = $('<div>修改行高</div>');
    $addLineHeight.css('cursor', 'pointer')
           .css('text-align', 'center')
           .css('border-bottom', '1px solid #e8e3e3')
           .css('padding', '5px 0');
    $menu.append($addLineHeight);
    $addLineHeight.on('click', () => {
       //  表单
       let lineheight = parseInt($border[0].getAttribute('data-blineheight'));
       lineheight = lineheight ? lineheight : 25;
       let $form = $('<form class="form"></form>');
       let $item1 = $('<div class="form-item"><span class="label">行高：</span><select name="blineheight">\
                        <option value="18" ' + ((lineheight === 18) ? 'selected' : '') + '>18px</option>\
                        <option value="20" ' + ((lineheight === 20) ? 'selected' : '') + '>20px</option>\
                        <option value="25" ' + ((lineheight === 25) ? 'selected' : '') + '>25px</option>\
                        <option value="30" ' + ((lineheight === 30) ? 'selected' : '') + '>30px</option>\
                        <option value="38" ' + ((lineheight === 38) ? 'selected' : '') + '>38px</option>\
                        <option value="44" ' + ((lineheight === 44) ? 'selected' : '') + '>44px</option>\
                        <option value="50" ' + ((lineheight === 50) ? 'selected' : '') + '>50px</option>\
                        </select></div>');
       $form.append($item1);
       // button 
       let $buttons = [];
       let $cancel = $('<span class="lsiten-message-button lsiten-message-cancel-button">取消</span>');
       $cancel.on('click', function () {
         message.hideMessage();
       })
       let $success = $('<span class="lsiten-message-button lsiten-message-success-button">确定</span>');
       $success.on('click', () => {
        let lineheightValue = $item1.find('select').val();
        $border.attr('data-blineheight', lineheightValue);
        $border.find('.js-lsiten-line').css('line-height', lineheightValue + 'px');
        Jquery($textElem[0]).trigger('addContent', () => {
          Jquery($textElem[0]).trigger('removeContent');
        });
        message.hideMessage();
       })
       $buttons.push($cancel);
       $buttons.push($success);
       // 2、打开对话框，填写修改的字数
       message.showMessage({options:{
          type: 'dialog',
          $buttons: $buttons,
          title: '框行高-修改',
          width: 500,
          content: $form
        }})
      $menu[0].blur();
    })
    // 添加文本框结束
    // 删除开始
    // let $delete = $('<div>删除框</div>');
    // $delete.css('cursor', 'pointer')
    //        .css('text-align', 'center')
    //        .css('padding', '5px 0');
    // $menu.append($delete);

    // $delete.on('click', () => {
    //   this._checkNeedBackStatus = true;
    //   // 如果是第一栏第一个框不允许删除
    //   let $currentParagraph = getParentByClassname($border, 'js-paragraph-view');
    //   let currentParagphChilds = $currentParagraph[0].childNodes;
    //   if (currentParagphChilds.length === 1 && $(currentParagphChilds[0]).equal($border)) {
    //     let $currentColumn = getParentByClassname($border, 'js-column');
    //     let $paragraphs = $currentColumn.find('.js-paragraph-view');
    //     let firstParagraph = this._cycleFindParagraph($paragraphs[0]);
    //     if (firstParagraph && $(firstParagraph).equal($currentParagraph)) {
    //       let pageIndex =  parseInt($currentColumn.attr('data-column-i'));
    //       if (this.pages.length === 1 && pageIndex === 0) {
    //         this.editor.message.showMessage('第一页第一个框不能删除！');
    //         $menu[0].blur();
    //         return false;
    //       }
    //     }
    //   }
    //   let $prevBorders = this._findPrevBorders($border);
    //   let $nextBorders = this._findNextBorders($border);
    //   let lastBorderIndex = $prevBorders.length - 1;
      
    //   $prevBorders.forEach(item => {
    //     let $item = $(item);
    //     let $paragraphItem = $item.parent();  
    //     $item.remove();
    //     if (isEmptyElement($paragraphItem)) {
    //       $paragraphItem.remove();
    //     }
    //   })

    //   $nextBorders.forEach(item => {
    //     let $item = $(item);
    //     let $paragraphItem = $item.parent();
    //     $item.remove();
    //     if (isEmptyElement($paragraphItem)) {
    //       $paragraphItem.remove();
    //     }
    //   })

    //   let $paragraphBorderItem = $border.parent();

    //   $border.remove();
    //   if (isEmptyElement($paragraphBorderItem)) {
    //     $paragraphBorderItem.remove();
    //   }
    //   $menu[0].blur();

    //   this._checkRemovePage();

    //   setTimeout(() => {
    //     this._checkoutContentChangeByFun();
    //   }, 100)
    // })

    // 删除结束
  },
 /**
 * function 获取前一栏
 */
  _getPrevColumn: function ($column) {
    let prev = $column[0].previousSibling;
    let pageIndex = parseInt($column.attr('data-column-page')) - 1;
    if (prev) {
      if (prev.className && prev.className.indexOf('js-column') > -1) {
        return $(prev);
      } else {
        if (pageIndex < 0) {
          return false;
        }

        let prevPage = this.pages[pageIndex];
        let lastColumnIndex = prevPage.$colums.length - 1;
        return prevPage.$colums[lastColumnIndex] || false;
      }
    } else {
      if (pageIndex < 0) {
        return false;
      }

      let prevPage = this.pages[pageIndex];
      let lastColumnIndex = prevPage.$colums.length - 1;
      return prevPage.$colums[lastColumnIndex] || false;
    }
  },
    /**
   * function 获取下一栏
   */
  _getNextColumn: function ($column) {
    let next = $column[0].nextSibling;
    let pageIndex = parseInt($column.attr('data-column-page')) + 1;
    if (next) {
      if (next.className && next.className.indexOf('js-column') > -1) {
        return $(next);
      } else {
        if (this.pages.length <= pageIndex) {
          return false;
        }
  
        let nextPage = this.pages[pageIndex];
        return nextPage.$colums[0] || false;
      }
    } else {
      if (this.pages.length <= pageIndex) {
        return false;
      }

      let nextPage = this.pages[pageIndex];
      return nextPage.$colums[0] || false;
    }
  },
  // 获得该框以上的框
  _findPrevBorders: function ($border) {
    let $paragraph = getParentByClassname($border, 'js-paragraph-view');
    let index = $paragraph.attr('data-index');
    // 如果上一段在同一栏，则返回空
    let prevParagraph = $paragraph[0].previousSibling;
    if (prevParagraph && prevParagraph.className && prevParagraph.className.indexOf('js-paragraph-view') > -1) {
      let firstContent = prevParagraph.firstChild;
      if (firstContent && firstContent.className && firstContent.className.indexOf('js-answer-header') < 0) {
        return [];
      }
    }


    // 获取上一栏的最后一段
    let $lastParagraph = this._getPrevColumnLastParagraph($paragraph);

    if (!$lastParagraph) {
      return [];
    }

    let prevIndex = $lastParagraph.attr('data-index');
    let prevLastParagraph = $lastParagraph.attr('class');


    if (index === prevIndex) {
      let borders = [];
      let lastBorder = $lastParagraph[0].lastChild;
      borders.push(lastBorder);
      if ($lastParagraph[0].childNodes.length === 1 && lastBorder.className && lastBorder.className.indexOf('js-lsiten-border') > -1 && prevLastParagraph.indexOf('js-split-paragraph') > -1) {
        let prevBorders = this._findPrevBorders($(lastBorder));
        borders = borders.concat(prevBorders);
      }
      return borders;
    } else {
      return [];
    }

  },
  // 获取该框以上的问题
  _findPrevQustions: function ($border, type) {
    if ( type === 4) {
      let $title = $border.find('.lsiten-title');
      if ($title.length) {
        return [];
      }
    }

    let $paragraph = getParentByClassname($border, 'js-paragraph-view');
    let index = $paragraph.attr('data-index');
    // 如果上一段在同一栏，则返回空
    let prevParagraph = $paragraph[0].previousSibling;
    if (prevParagraph && prevParagraph.className && prevParagraph.className.indexOf('js-paragraph-view') > -1) {
      let firstContent = prevParagraph.firstChild;
      if (firstContent && firstContent.className && firstContent.className.indexOf('js-answer-header') < 0) {
        return [];
      }
    }


    // 获取上一栏的最后一段
    let $lastParagraph = this._getPrevColumnLastParagraph($paragraph);

    if (!$lastParagraph) {
      return [];
    }

    let prevIndex = $lastParagraph.attr('data-index');
    let prevLastParagraph = $lastParagraph.attr('class');

    if (index === prevIndex) {
      let lastBorder = $lastParagraph[0].lastChild;
      let borderContent = lastBorder.firstChild;
      if (borderContent) {
        let lastQuestion = borderContent.lastChild;
        let questionType = parseInt(lastQuestion.getAttribute('data-type'));
        let questions = [];
        if (type === questionType) {
          questions.push(lastQuestion);
  
          if (borderContent.childNodes.length === 1 && prevLastParagraph.indexOf('js-split-paragraph') > -1) {
            let prevQuestions = this._findPrevQustions($(lastBorder), type);
            questions = questions.concat(prevQuestions);
          }
          return questions;
        } else {
          return [];
        }
      } else {
        return [];
      }

    } else {
      return [];
    }
    
  },
  // 获取上一栏的最后一段
  _getPrevColumnLastParagraph: function ($paragraph) {
      let $colum = getParentByClassname($paragraph, 'js-column');
      if (!$colum) {
        return null;
      }
  
      let $prevColumn = this._getPrevColumn($colum);
  
      if (!$prevColumn) {
        return null;
      }
  
      let $paragraphs = $prevColumn.find('.js-paragraph-view');
  
      if (!$paragraphs.length) {
        return null;
      }
  
      let lastIndex = $paragraphs.length - 1;
  
      let lastParagraph = $paragraphs[lastIndex];
  
      let first = lastParagraph.firstChild;
      if (first.className && first.className.indexOf('js-answer-header') > -1) {
        return null;
      }
  
      return $(lastParagraph);
  
    },
  
  _findNextBorders: function ($border) {
      let $paragraph = getParentByClassname($border, 'js-paragraph-view');
      let index = $paragraph.attr('data-index');
      // 如果上一段在同一栏，则返回空
      let nextParagraph = $paragraph[0].nextSibling;
      if (nextParagraph && nextParagraph.className && nextParagraph.className.indexOf('js-paragraph-view') > -1) {
        let firstContent = nextParagraph.firstChild;
        if (firstContent && firstContent.className && firstContent.className.indexOf('js-answer-header') < 0) {
          return [];
        }
      }
  
      // 获取下一栏的第一段
      let $firstParagraph = this._getNextColumnfirstParagraph($paragraph);
  
      if (!$firstParagraph) {
        return [];
      }
  
      let nextIndex = $firstParagraph.attr('data-index');
  
      let nextFirstParagraph = $firstParagraph.attr('class');
  
      if (index === nextIndex) {
        let borders = [];
        let firstBorder = $firstParagraph[0].firstChild;
        borders.push(firstBorder);
        if ($firstParagraph[0].childNodes.length === 1 && firstBorder.className && firstBorder.className.indexOf('js-lsiten-border') > -1 && nextFirstParagraph.indexOf('js-split-paragraph') > -1) {
          let nextBorders = this._findNextBorders($(firstBorder));
          borders = borders.concat(nextBorders);
        }
        return borders;
      } else {
        return [];
      }
  },
  // 获取该框以下的问题
  _findNextQustions: function ($border, type) {
      let $paragraph = getParentByClassname($border, 'js-paragraph-view');
      let index = $paragraph.attr('data-index');
      // 如果上一段在同一栏，则返回空
      let nextParagraph = $paragraph[0].nextSibling;
      if (nextParagraph && nextParagraph.className && nextParagraph.className.indexOf('js-paragraph-view') > -1) {
        let firstContent = nextParagraph.firstChild;
        if (firstContent && firstContent.className && firstContent.className.indexOf('js-answer-header') < 0) {
          return [];
        }
      }
  
      // 获取下一栏的第一段
      let $firstParagraph = this._getNextColumnfirstParagraph($paragraph);
  
      if (!$firstParagraph) {
        return [];
      }
  
      let nextIndex = $firstParagraph.attr('data-index');
  
      let nextFirstParagraph = $firstParagraph.attr('class');
  
      if (index === nextIndex) {
        let firstBorder = $firstParagraph[0].firstChild;
        let borderContent = firstBorder.firstChild;
        if (borderContent) {
          let firstQuestion = borderContent.firstChild;
          let questionType = parseInt(firstQuestion.getAttribute('data-type'));
          let questions = [];
          if (type === questionType) {
            questions.push(firstQuestion);
    
            if (borderContent.childNodes.length === 1 && nextFirstParagraph.indexOf('js-split-paragraph') > -1) {
              let nextQuestions = this._findNextQustions($(firstBorder), type);
              questions = questions.concat(nextQuestions);
            }
            return questions;
          } else {
            return [];
          }
        } else {
          return [];
        }
  
      } else {
        return [];
      }
  },
  
  // 获取下一栏的第一段
  _getNextColumnfirstParagraph: function ($paragraph) {
      let $colum = getParentByClassname($paragraph, 'js-column');
      if (!$colum) {
        return null;
      }
  
      let $nextColumn = this._getNextColumn($colum);
  
      if (!$nextColumn) {
        return null;
      } 
  
      let $paragraphs = $nextColumn.find('.js-paragraph-view');
  
      if (!$paragraphs.length) {
        return null;
      }
  
      let firstParagraph = this._cycleFindParagraph($paragraphs[0]);
      if (!firstParagraph) {
        return null;
      }
  
      let firstContent = firstParagraph.firstChild;
      if (firstContent && firstContent.className && firstContent.className.indexOf('js-answer-header') > -1) {
        return null;
      }
  
      return $(firstParagraph);
  },
  
  
  // 循环查找段落 
  _cycleFindParagraph: function (paragraph) {
      if (!paragraph) {
        return null;
      }
      let i = 0;
      while (i < 100 && (!paragraph.className || paragraph.className.indexOf('js-paragraph-view') < 0)) {
        let temp = paragraph.nextSibling;
        paragraph.parentNode.removeChild(paragraph);
        paragraph = temp;
         i++;
      }
  
  
      let firstContent = paragraph.firstChild;
      if (firstContent && firstContent.className && firstContent.className.indexOf('js-answer-header') > -1) {
        let temp = paragraph.nextSibling;
        paragraph = temp; 
        return this._cycleFindParagraph(paragraph);
      }
       return paragraph || null;
  },
   /** 
   * function 内容变化事件监听
  */
  _contentchangeEvent: function () {
    const $textElem = this.$el;
    const editor =this.editor;
    let _this = this;
    Jquery($textElem[0]).on('addContent', function (e, callback) {
      _this._checkEveryPageOut(callback);
    })

    Jquery($textElem[0]).on('removeContent', function (e, callback) {
      _this._checkEveryPageBack(callback);
    })
  },
  /** 
   * function del键监听
  */
  _delEvent: function () {
    const $textElem = this.$el;
    const editor =this.editor;
    let _this = this;
    Jquery($textElem[0]).on('keydown', function (e) {    
      if (e.keyCode !== 46) {
        // 不是回车键
        return '';
      }
      if (_this._$handleImg) {
        _this._$handleImg.remove();
        _this._$handleImg = null;
        _this._$drageDiv.hide();
        e.preventDefault();
        return '';
      }
      if (getParentByClassname($(e.target), 'js-answer-header-title')) {
        return '';
      }
      if (getParentByClassname($(e.target), 'lsiten-e-message-body-content')) {
        return '';
      }
      let $currentLine = _this.editorPosition.currentLine;
      if ($currentLine) {
        let selection = _this.editor.selection;
        if (selection.isCartEnd($currentLine)) {
          let nextLine = $currentLine[0].nextSibling;
          if (nextLine && nextLine.className && nextLine.className.indexOf('js-lsiten-line') > -1 && isEmptyElement(nextLine)) {
            $(nextLine).remove();
            Jquery($textElem[0]).trigger('removeContent');
            return false;          
          } else {
            return false;
          }
        }
      } else {
        return false;
      }
    })
  },
  /** 
   * function 退格键监听
  */
  _backEvent () {
    const $textElem = this.$el;
    const editor =this.editor;
    let _this = this;
    Jquery($textElem[0]).on('keydown', function (e) {       
      if (e.keyCode !== 8) {
        // 不是回车键
        return '';
      }
      if (_this._$handleImg) {
        _this._$handleImg.remove();
        _this._$handleImg = null;
        _this._$drageDiv.hide();
        e.preventDefault();
        return '';
      }
      if (getParentByClassname($(e.target), 'js-answer-header-title')) {
        return '';
      }

      if (getParentByClassname($(e.target), 'lsiten-e-message-body-content')) {
        return '';
      }
      let $currentLine = _this.editorPosition.currentLine;
      if ($currentLine) {
        if (isEmptyElement($currentLine[0])) {
          let prevLine = $currentLine[0].previousSibling;
          if (prevLine && prevLine.className && prevLine.className.indexOf('js-lsiten-line') > -1) {
            $currentLine.remove();
            prevLine.focus();
            Jquery($textElem[0]).trigger('removeContent');
            return false;            
          } else {
            return false;
          }
        }
      } else {
        return false;
      }
    })
  },
  /**
   * function 监听enter键事件
   */
  _enterEvent: function () {
    const $textElem = this.$el;
    const editor =this.editor;
    let _this = this;
    Jquery($textElem[0]).on('keydown', function (e) {       
      if (e.keyCode !== 13) {
        // 不是回车键
        return '';
      }
      if (getParentByClassname($(e.target), 'js-answer-header-title')) {
        return '';
      }
      let $currentLine = _this.editorPosition.currentLine;
      if ($currentLine) {
        let $newLine = gennerOneLine();
        let $borderContent = getParentByClassname($currentLine, 'js-lsiten-border');
        if ($borderContent) {
          let lineHeight = $borderContent[0].getAttribute('data-blineheight');
          $newLine = gennerOneLine(false, lineHeight);
        }
        $newLine.insertAfter($currentLine);
        $newLine.focus();
      }

      Jquery($textElem[0]).trigger('addContent');
      return false;
    })
  },
  /**
   * function 实时保存当前编辑信息
   */
  _realtimeSavePosition: function () {
    const $textElem = this.$el;
    const editor =this.editor;
    let _this = this;
    function realTimeSave (e) {
      _this._updatePositionInfo(e);
    }

    Jquery($textElem[0]).on('compositionend', function () {       
      _this.inputType = false;
    })
    Jquery($textElem[0]).on('keyup',  realTimeSave);
    Jquery($textElem[0]).on('keydown',  realTimeSave);
    Jquery($textElem[0]).on('mousedown',  e => {
      // mousedown 状态下，鼠标滑动到编辑区域外面，也需要保存选区
      Jquery($textElem[0]).on('mouseleave', '.js-column', realTimeSave);
    })
    Jquery($textElem[0]).on('mouseup', '.js-column', e => {
        realTimeSave(e);
        // 在编辑器区域之内完成点击，取消鼠标滑动到编辑区外面的事件
        Jquery($textElem[0]).off('mouseleave', realTimeSave);
    })
  },

  // 作文题处理
  _writtingHandleEvent: function () {
    // 为图片增加 selected 样式
    Jquery(this.editor.$editorArea[0]).on('contextmenu','.lsiten-question-writing', (e) => {
      let ele = e.target || e.srcElement;
      let message = this.editor.message;
      let _this = this;
      let writting = getParentNodeByClass(ele, 'lsiten-question-writing');
      if (writting) {
        let $writting = $(writting);
        let menus = [
          {
            name: "修改",
            click: function ($el) {
              $el.on('click', () => {
                let pnum = $writting.attr('data-pnum');
                let total = parseInt($writting.attr('data-total'));
                //  表单
                let $form = $('<form class="form"></form>');
                let $item1 = $('<div class="form-item"><span class="label">题号：</span><span>' + pnum + '</span></div>');
                let $item2 = $('<div class="form-item"><span class="label">格子数：</span><input value = " ' + total + ' " type="text" placeholder="请输入总共多少个格子" class="block"/></div>');
                $form.append($item1);
                $form.append($item2);
                // button 
                let $buttons = [];
                let $cancel = $('<span class="lsiten-message-button lsiten-message-cancel-button">取消</span>');
                $cancel.on('click', function () {
                  message.hideMessage();
                })
                let $success = $('<span class="lsiten-message-button lsiten-message-success-button">确定</span>');

                function sureClick () {
                  message.hideMessage();
                  let total = parseInt($item2.find('input').val());
                  if (total > 3000 || total < 1) {
                    message.showMessage('错误：字数应在1-3000内');
                    return false;
                  }
                  // 重新生成作文题框

                  // 找到整个题框，并删除
                  //   1、判断是否是跨栏框
                  let writingClass = $writting.attr('class');

                  if (writingClass.indexOf('js-split-question') > -1) {
                    let $prevQuestions = _this._findPrevQustions($writting, 4);
                    let $nextQuestions = _this._findNextQustions($writting, 4);
                    let lastIndex = $prevQuestions.length - 1;
                    let questionBasic = lastIndex >= 0 ? $prevQuestions[lastIndex] : $writting[0];
                    let $borderBasic = $(questionBasic).parent();
                    let lineHeight = $borderBasic.parent()[0].getAttribute('data-blineheight');
                    $prevQuestions.forEach(item => {
                      let $item = $(item);
                      let $borderContent = $item.parent();
                      $item.remove();
                      if (isEmptyElement($borderContent)) {
                        let $border = $borderContent.parent();
                        if (!$borderBasic.equal($borderContent)) {
                          let $paragraphTemp = $border.parent();
                          $border.remove();
                          if (isEmptyElement($paragraphTemp)) {
                            $paragraphTemp.remove();
                          }
                        }
                      }
                    })
                    $nextQuestions.forEach(item => {
                      let $item = $(item);
                      let $borderContent = $item.parent();
                      $item.remove();
                      if (isEmptyElement($borderContent)) {
                        let $paragraphTemp = $borderContent.parent().parent();
                        $borderContent.parent().remove();
                        if (isEmptyElement($paragraphTemp)) {
                          $paragraphTemp.remove();
                        }
                      }
                    })

                    let $borderWrittingContent = $writting.parent();
                    $writting.remove();
                    if (isEmptyElement($borderWrittingContent)) {
                      let $borderWritting = $borderWrittingContent.parent()
                      if (!$borderBasic.equal($borderWrittingContent)) {
                        let $paragraphTemp = $borderWritting.parent();
                        $borderWritting.remove();
                        if (isEmptyElement($paragraphTemp)) {
                          $paragraphTemp.remove();
                        }
                      }
                    }

                    _this._checkRemovePage();
                    let $onelineBasic = gennerOneLine(false, lineHeight);
                    $borderBasic.append($onelineBasic);
                    _this._updatePositionInfo(null, $onelineBasic);
                  } else {
                    let $borderWrittingContent = $writting.parent();
                    $writting.remove();
                    let $onelineBasic = gennerOneLine(false, lineHeight);
                    $borderWrittingContent.append($onelineBasic);
                    _this._updatePositionInfo(null, $onelineBasic);
                  }

                  _this.editor.cmd.do('questiontype', {type: 4, data: {
                      pnum: pnum,
                      nums: total,
                      visible: true
                  }});
                }
                $success.on('click', function () {
                  sureClick();
                })
                Jquery(document).on('keydown', function(event) {   
                  if (event.keyCode !== 13) {
                    // 不是回车键
                    return '';
                  }                          
                  if (Jquery(message.$el[0]).css('display') === 'block') {
                    sureClick();
                    event.preventDefault();
                    return true;
                  }
                })
                $buttons.push($cancel);
                $buttons.push($success);
                // 1、关闭右击menu
                let $contentMenu = getParentByClassname($el, 'js-lsiten-contentmenus');
                // 2、打开对话框，填写修改的字数
                message.showMessage({options:{
                  type: 'dialog',
                  $buttons: $buttons,
                  title: '作文题-修改',
                  width: 500,
                  content: $form
                }})
                if ($contentMenu.length) {
                  try {
                    $contentMenu.remove();
                  } catch (error) {
                    // console.log(error);
                  }
                }
              })
            }
          }
        ]
        this._createContextMenu(e, menus);
        e.stopImmediatePropagation();
      } else {
        console.log('c%error: no writting box', 'color:red');
      }
    })
  },

  // 滚动事件监听 
  _scrollEvent: function () {
    let _this = this;
    Jquery(this.editor.$editorArea[0]).on('scroll', () => {
      if (_this._$handleImg) {
        _this._$handleImg = null;
        _this._$drageDiv.hide();
      }
    })
  },
  // 滚动事件监听 
  _resizeEvent: function () {
    let _this = this;
    Jquery(window).on('resize', () => {
      if (_this._$handleImg) {
        _this._$handleImg = null;
        _this._$drageDiv.hide();
      }
    })
  },


  _gennerateDragBox: function ($dom) {
    let position = $dom.getSizeData();
    let $drag = this._$drageDiv;
    $drag.css('left', (position.x - 3) + 'px')
         .css('top', (position.y - 3) + 'px')
         .css('width', (position.width + 6) + 'px')
         .css('height', (position.height + 6) + 'px');
    $drag.show();
  },
  // 图片处理
  _imgHandle: function () {
    const $textElem = this.$el;
    const _this = this;
    const editor = this.editor;
    Jquery($textElem[0]).on('dragstart, dragend', 'img', function (e) {
      e.preventDefault();
    })
    // 为图片增加 selected 样式
    $textElem.on('click', 'img', function (e) {
      let ele = e.target || e.srcElement;
      let $img = $(ele);
      if (ele.className && ele.className.indexOf('lsiten-js-image-class') > -1) {
        _this._$handleImg = $img;
        _this._gennerateDragBox($img);
      }
    })
    // 去掉图片的 selected 样式
    $textElem.on('click  keyup', e => {
      if (e.target.matches('img')) {
        // 点击的是图片，忽略
        return '';
      }
      let $drag = this._$drageDiv;
      _this._$handleImg = null;
      $drag.hide();
    })
    // 粘贴图片、文本的事件，每次只能执行一个
    // 判断该次粘贴事件是否可以执行
    const config = editor.config;
    const pasteFilterStyle = config.pasteFilterStyle;
    const pasteTextHandle = config.pasteTextHandle;
    const ignoreImg = config.pasteIgnoreImg;
    let pasteTime = 0
    function canDo() {
        var now = Date.now()
        var flag = false
        if (now - pasteTime >= 100) {
            // 间隔大于 100 ms ，可以执行
            flag = true
        }
        pasteTime = now
        return flag
    }

    function resetTime() {
        pasteTime = 0
    }

    // 粘贴文字
    $textElem.on('paste', e => {
        if (UA.isIE()) {
            return
        } else {
            // 阻止默认行为，使用 execCommand 的粘贴命令
            e.preventDefault()
        }

        // 粘贴图片和文本，只能同时使用一个
        if (!canDo()) {
            return
        }

        // 获取粘贴的文字
        let pasteHtml = getPasteHtml(e, pasteFilterStyle, ignoreImg)
        let pasteText = getPasteText(e)
        pasteText = pasteText.replace(/\n/gm, '<br>')

        const $selectionElem = editor.selection.getSelectionContainerElem()
        if (!$selectionElem) {
            return
        }
        if (!pasteHtml) {
            // 没有内容，可继续执行下面的图片粘贴
            resetTime()
            return
        }
        try {
            // firefox 中，获取的 pasteHtml 可能是没有 <ul> 包裹的 <li>
            // 因此执行 insertHTML 会报错
            if (pasteTextHandle && isFunction(pasteTextHandle)) {
                // 用户自定义过滤处理粘贴内容
                pasteHtml = '' + (pasteTextHandle(pasteHtml) || '')
            }
            editor.cmd.do('insertHTML', pasteHtml)
        } catch (ex) {
            // 此时使用 pasteText 来兼容一下
            if (pasteTextHandle && isFunction(pasteTextHandle)) {
                // 用户自定义过滤处理粘贴内容
                pasteText = '' + (pasteTextHandle(pasteText) || '')
            }
            editor.cmd.do('insertHTML', `<p>${pasteText}</p>`)
        }
    })

    // 粘贴图片
    $textElem.on('paste', e => {
        if (UA.isIE()) {
            return '';
        } else {
            e.preventDefault();
        }

        // 粘贴图片和文本，只能同时使用一个
        if (!canDo()) {
            return '';
        }

        // 获取粘贴的图片
        const pasteFiles = getPasteImgs(e);
        if (!pasteFiles || !pasteFiles.length) {
            return '';
        }

        // 获取当前的元素
        const $currentLine = this.editorPosition.currentLine;
        if (!$currentLine) {
            return '';
        }

        // 上传图片
        _this._uploadFiles(pasteFiles);
    })

    
  },

  /**
   * function 上传图片
   * @param {array} files 图片兑现数组 
   */
  _uploadFiles: function (files) {
    if (!files || !files.length) {
        return '';
    }
    const editor = this.editor;
    let policyUrl = editor.config.policyUrl;
    const message = editor.message;
    // 获取当前的元素
    const $currentLine = this.editorPosition.currentLine;
    if (!$currentLine) {
        return '';
    }
    let borderText = getParentByClassname($currentLine, 'js-lsiten-border');
    let size = null;
    if (borderText) {
      size = borderText.getSizeData();
    } else {
      return '';
    }
    
    arrForEach(files, file => {
      uploadToAliyun(file, policyUrl).then(data => {
        if (data.result === '00') {
          let src = data.data.url;
          let srcStore = data.data.urlStore;
          this._doInsertImage({
            src: src,
            store: srcStore,
            bsize: size,
            border: borderText
          }); 
        } else {
           message.showMessage(data.msg || '上传出错，请重试！');
        }
     })
    })

  },
  _doInsertImage: function (img) {
    let imageTool = new imgUtl(img.src);
    let _this = this;
    imageTool.getSize(complete);
    function complete() {
      let owidth = imageTool.owidth;
      let bsize = img.bsize;
      let bwidth = bsize.width - 30;
      if (owidth >= bwidth) {
        let ratio = bwidth / owidth;
        imageTool.resizeByRatio(ratio);
      }
      let oheight = imageTool.size.h;
      let bheight = bsize.height - 22;
      if (oheight > bheight) {
        let ratio = bheight / oheight;
        imageTool.resizeByRatio(ratio);
      }
      let html = $('<img class="lsiten-js-image-class" style="width: ' + imageTool.size.w + 'px; height: ' + imageTool.size.h + 'px" src="' + img.src + '" data-store = "' +img.store+ '"/>');
      html.css('position', 'absolute').css('top', '0').css('left', '0');
      img.border.append(html);      
    }
  },
  // 初始化拖拽框
  _initDrageDiv: function () {
    this._$drageDiv = $('<div class="js-lsiten-dragbox"></div>');
    this._$drageDiv.css('height', '0')
                   .css('width', '0')
                   .css('position', 'fixed')
                   .css('z-index', '9998')
                   .css('cursor', 'move')
                   .css('display', 'none')                   
                   .css('border', '1px dashed #5b3838');
    let $delete = $('<div class="js-lsiten-dragbox-delete">删除</div>');
    $delete.css('height', '14px')
         .css('width', 'auto')
         .css('cursor', 'pointer')
         .css('right', '-20px')
         .css('font-size', '12px')
         .css('bottom', '-20px')
         .css('position', 'absolute');
    let $left = $('<div class="js-lsiten-dragbox-left"></div>');
    $left.css('height', '14px')
         .css('width', '14px')
         .css('background', '#e1dcdc')
         .css('cursor', 'nw-resize')
         .css('left', '-7px')
         .css('top', '-7px')
         .css('position', 'absolute');
    
    let $leftBottom = $('<div class="js-lsiten-dragbox-left-bottom"></div>');
    $leftBottom.css('height', '14px')
         .css('width', '14px')
         .css('background', '#e1dcdc')
         .css('left', '-7px')
         .css('bottom', '-7px')
         .css('cursor', 'sw-resize')
         .css('position', 'absolute');

    let $right = $('<div class="js-lsiten-dragbox-right"></div>');
    $right.css('height', '14px')
         .css('width', '14px')
         .css('background', '#e1dcdc')
         .css('right', '-7px')
         .css('top', '-7px')
         .css('cursor', 'ne-resize')
         .css('position', 'absolute');

    
    let $rightBottom = $('<div class="js-lsiten-dragbox-right-bottom"></div>');
    $rightBottom.css('height', '14px')
         .css('width', '14px')
         .css('background', '#e1dcdc')
         .css('right', '-7px')
         .css('bottom', '-7px')
         .css('cursor', 'se-resize')
         .css('position', 'absolute');
    this._$drageDiv.append($left);
    this._$drageDiv.append($right);
    this._$drageDiv.append($leftBottom);
    this._$drageDiv.append($rightBottom);
    this._$drageDiv.append($delete);
    
    let _this = this;
    let templatePosition = {
      x: 0,
      y: 0,
      type: 0
    }
    $delete.on('click', () => {
      if (_this._checkCrossing || _this._contentChangeStatus) {
        _this._$handleImg = null;
        _this._$drageDiv.hide();
        return false;
      }
      let $img = _this._$handleImg;
      if ($img) {
        $img.remove();
        _this._$handleImg = null;
        _this._$drageDiv.hide();
        return true;
      } else {
        _this._$handleImg = null;
        _this._$drageDiv.hide();
        return false;
      }
    })
    this._$drageDiv.on('mousedown', function (e) {
      templatePosition = {
        type: 5,
        x: e.pageX,
        y: e.pageY,
        oPosition: {
          x: e.pageX,
          y: e.pageY,
        }
      }
      e.preventDefault();
    })
    $delete.on('mousedown', function (e) {
      e.stopPropagation();
    })
    $left.on('mousedown', function (e) {
      templatePosition = {
        type: 1,
        x: e.pageX,
        y: e.pageY,
        oPosition: {
          x: e.pageX,
          y: e.pageY,
        }
      }
      e.stopPropagation();
    })

    $leftBottom.on('mousedown', function (e) {
      templatePosition = {
        type: 2,
        x: e.pageX,
        y: e.pageY,
        oPosition: {
          x: e.pageX,
          y: e.pageY,
        }
      }
      e.stopPropagation();      
    })

    $right.on('mousedown', function (e) {
      templatePosition = {
        type: 3,
        x: e.pageX,
        y: e.pageY,
        oPosition: {
          x: e.pageX,
          y: e.pageY,
        }
      }
      e.stopPropagation();      
    })

    $rightBottom.on('mousedown', function (e) {
      templatePosition = {
        type: 4,
        x: e.pageX,
        y: e.pageY,
        oPosition: {
          x: e.pageX,
          y: e.pageY,
        }
      }
      e.stopPropagation();      
    })
    $(document).on('mousemove', function (e) {
      if (templatePosition.type === 0) {
        return false;
      }
      let diffX = parseInt(e.pageX - templatePosition.x);
      let diffY = parseInt(e.pageY - templatePosition.y);
      let borderSize = _this._$drageDiv.getSizeData();

      switch (templatePosition.type) {
        case 1:

        let bleft = parseInt(borderSize.left) + diffX;
        let btop = parseInt(borderSize.top) + diffY;
        let width = parseInt(borderSize.width) - diffX;
        let height = parseInt(borderSize.height) - diffY;
        if (width < 0 || height < 0) {
          return false;
        }
        templatePosition.x = e.pageX;
        templatePosition.y = e.pageY;
        _this._$drageDiv.css('left', bleft + 'px').css('top', btop + 'px').css('width', width + 'px').css('height', height + 'px');
        break;
        case 2:

        let bleft2 = parseInt(borderSize.left) + diffX;
        let bbottom2 = parseInt(borderSize.bottom) - diffY;
        let width2 = parseInt(borderSize.width) - diffX;
        let height2 = parseInt(borderSize.height) + diffY;
        if (width2 < 0 || height2 < 0) {
          return false;
        }
        templatePosition.x = e.pageX;
        templatePosition.y = e.pageY;
        _this._$drageDiv.css('left', bleft2 + 'px').css('bottom', bbottom2 + 'px').css('width', width2 + 'px').css('height', height2 + 'px');
        break;
        case 3:
        let bright = parseInt(borderSize.right) - diffX;
        let btop3 = parseInt(borderSize.top) + diffY;
        let width3 = parseInt(borderSize.width) + diffX;
        let height3 = parseInt(borderSize.height) - diffY;
        if (width3 < 0 || height3 < 0) {
          return false;
        }
        templatePosition.x = e.pageX;
        templatePosition.y = e.pageY;
        _this._$drageDiv.css('right', bright + 'px').css('top', btop3 + 'px').css('width', width3 + 'px').css('height', height3 + 'px');
        break;
        case 4:
        let bright2 = parseInt(borderSize.right) - diffX;
        let bbottom4 = parseInt(borderSize.top) - diffY;
        let width4 = parseInt(borderSize.width) + diffX;
        let height4 = parseInt(borderSize.height) + diffY;
        if (width4 < 0 || height4 < 0) {
          return false;
        }
        templatePosition.x = e.pageX;
        templatePosition.y = e.pageY;
        _this._$drageDiv.css('right', bright2 + 'px').css('bottom', bbottom4 + 'px').css('width', width4 + 'px').css('height', height4 + 'px');
        break;
        case 5:
        let bx = parseInt(borderSize.left) + diffX;
        let by = parseInt(borderSize.top) + diffY;
        templatePosition = {
          type: 5,
          x: e.pageX,
          y: e.pageY,
          oPosition: templatePosition.oPosition
        }
        _this._$drageDiv.css('left', bx + 'px').css('top', by + 'px');
        break;
      }
    })
    $(document).on('mouseup', function (e) {
      if (templatePosition.type === 0) {
        return false;
      }
      if (_this._checkCrossing || _this._contentChangeStatus) {
        _this._$handleImg = null;
        _this._$drageDiv.hide();
        return false;
      }
      let $img = _this._$handleImg;

      if ($img) {
        let $border = getParentByClassname($img, 'js-lsiten-border');
        let borderBoxSize = $border.getSizeData();
        let borderImgBoxSize = $img.getSizeData();
        let $drag = _this._$drageDiv;        
        let borderSize = $drag.getSizeData();
        if (templatePosition.type === 5) {
          // debugger;
          let imageLeft = parseInt(borderSize.left - borderBoxSize.left);
          let imageTop =  parseInt(borderSize.top - borderBoxSize.top);
          imageLeft = (imageLeft + borderImgBoxSize.width + 4) > borderBoxSize.width ? (borderBoxSize.width - borderImgBoxSize.width -4) : imageLeft;
          imageTop = (imageTop + borderImgBoxSize.height + 4) > borderBoxSize.height ? (borderBoxSize.height - borderImgBoxSize.height -4) : imageTop;

          imageLeft < 0 && (imageLeft = 0);
          imageTop < 0 && (imageTop = 0);
          $img.css('left', imageLeft + 'px').css('top', imageTop + 'px');
        } else {
          let width = parseInt(borderSize.width) - 6;
          let prevHeight = _this.getPrevHeight($img, $border);
          let height = parseInt(borderSize.height) - 6;
          let borderHeight = borderBoxSize.height - prevHeight;
          let borderwidth = borderBoxSize.width;
          width = width > borderwidth ? (borderwidth - 30) : width;
          height = height > borderHeight ? (borderHeight - 22) : height;
          $img.css('width', width + 'px').css('height', height + 'px');
        }


        let position = $img.getSizeData();
        $drag.css('left', (position.x - 3) + 'px')
            .css('top', (position.y - 3) + 'px');
        let $handleImg = _this._$handleImg;
        _this._$handleImg = null;
        _this._$drageDiv.hide();
        $handleImg[0].click();
      }

      templatePosition = {
        x: 0,
        y: 0,
        type: 0
      }
    })
    this.editor.$container.append(this._$drageDiv);
  },
  // 获取dom元素前面的高度
  getPrevHeight: function ($dom, $border) {
    let domSize = $dom.getSizeData();
    let borderSize = $border.getSizeData();
    let diff = domSize.top - borderSize.top;
    return diff > 0 ? diff : 0;
  },
  /**
   * 更新当前编辑信息
   */
  _updatePositionInfo: function (e, $ele) {
    let $target = $ele || $(e.target || e.toElement);
    if (!$target) {
      return false;
    }
    let $border = getParentByClassname($target, 'js-lsiten-border');
    let $line = getParentByClassname($target, 'js-lsiten-line');
    let $paragraph = getParentByClassname($target, 'js-paragraph-view');
    if (!$paragraph) {
      return false;
    }

    let $currentColumn = getParentByClassname($paragraph, 'js-column');
    if (!$currentColumn) {
      console.log($target, console.log("%c数据错误！", "color:red"));
      return false;
    }
    this.currentPage = this.pages[parseInt($currentColumn.attr('data-column-page'))];
    if (!this.currentPage) {
      console.log($target, $currentColumn, $paragraph, console.log("%c数据错误！", "color:red"));
      return false;
    }
    this.currentPage.currentColumn = parseInt($currentColumn.attr('data-column-i'));

    this.editorPosition = {
      currentParagraph: $paragraph,
      currentColumn: this.currentPage.$colums[this.currentPage.currentColumn],
      currentPage: this.currentPage,
      currentBorder: $border,
      currentLine: $line
    }

    let selection = this.editor.selection;
    if ($line) {
      if (!selection.getRange() ||  !selection.getSelectionContainerElem()) {
        selection.createRangeByElem($line, true);
      } else {
        selection.saveRange();
      }
    } else {
      selection.setEmptyRange();
    }
    this.editor.menus.changeActive();
  },
  /**
   * function 拖拽事件
   */
  _dragEvent: function () {
    // 禁用 document 拖拽事件
    const $document = $(document)
    $document.on('dragleave drop dragenter dragover', function (e) {
        e.preventDefault()
    })
  },
  _pagesWidth: {
    'A4': '793px',
    'A3': '1587px'
  },
  _pagesHeight: {
    'A4': '1122px',
    'A3': '1122px'
  },
  _pagesBreakHeight: {
    'A4': '1122',
    'A3': '1122'
  }
}
export default yzPageManager
