import '../less/scroller.less';
import '../less/common.less';
import '../less/icon.less';
import '../less/main.less';

import $ from './util/dom-core.js';
import _config from '../config.js';
import Command from './command/index.js';
import Menus from './menus/index.js';
import Message from './util/message.js';
import selectionAPI from './selection/index.js';
import { arrForEach, objForEach } from './util/util.js';
import { getRandom } from './util/util.js';

import yzPageManager from './yzPageManager';

 
let yzWebeditor = function (cfg) {
  this.$container = $(cfg.dom) || $(document.body);
  this.$editorArea = $('<div class="lsiten-editor-view"></div>')
  this.data = cfg.data || {}; // 数据源
  this.page = null;
  this.pageSize = this.data.paperSize || 'A3';
  this.columnNumber = parseInt(this.data.layoutType) || 2;
  this.noMode = parseInt(this.data.noMode) || 2;
  this.noCount = parseInt(this.data.noCount) || 12;
  this.layoutMode = parseInt(this.data.layoutMode) || 1;

  this.cardHtml = this.data.cardHtml || null;
  this.cardHtml && (this.cardHtml = JSON.parse(this.cardHtml));

  this.editorHeight = 800;
  this.customConfig = cfg.config || {};
  this.id = ('lsiten_editor_'+Math.random()).replace('.','_');
  this.isDoObject = false;
  this._init();
}

yzWebeditor.prototype = {
  constructor: yzWebeditor,
  // 初始化页面
  _init: function () {
    this.$container.addClass('lsiten-container');
    this._initConfig();
    this._initEditStyle();
    this._initCommand();
    this._initSelectionAPI();
    this._initToolbar();
    this._addPage();
    this.$container.append(this.$toolbar);
    this.$container.append(this.$editorArea);
    this.initSelection(true);
    this._initMessage();
    this._bindEvent();
  },
  _initToolbar: function() {
    this.$toolbar = $('<div class="lsiten-editor-toolbar"></div>');
    this.menus = new Menus(this);
    this.menus.init();
  },
  _initMessage: function () {
    this.message = new Message(this, {});
  },
  _initEditStyle: function() {
    this.$editorArea.css('height', this.editorHeight + 'px').css('overflow', 'scroll');
  },
  // 初始化配置文
  _initConfig: function () {
     // _config 是默认配置，this.customConfig 是用户自定义配置，将它们 merge 之后再赋值
     let target = {};
     this.config = Object.assign(target, _config, this.customConfig);

     // 将语言配置，生成正则表达式
     const langConfig = this.config.lang || {};
     const langArgs = [];
     objForEach(langConfig, (key, val) => {
         // key 即需要生成正则表达式的规则，如“插入链接”
         // val 即需要被替换成的语言，如“insert link”
         langArgs.push({
             reg: new RegExp(key, 'img'),
             val: val
         })
     });
     this.config.langArgs = langArgs;
  },
  // 添加页面 
  _addPage: function() {
    this.page = new yzPageManager(this);
    this.$editorArea.append(this.page.$el);
  },
  // 封装 command API
  _initCommand: function () {
    this.cmd = new Command(this);
  },
  // 封装 selection range API
  _initSelectionAPI: function () {
    this.selection = new selectionAPI(this);
  },
  // 初始化选区，将光标定位到内容尾部
  // @todo
  initSelection: function (newLine) {
    const currentPage = this.page.currentPage;
    if (currentPage) {
      const $currentColumn = currentPage.$colums[currentPage.currentColumn];
      const $children = $currentColumn.children();
      if (!$children.length && !$currentColumn.html()) {
          // 如果编辑器区域无内容，添加一个空行，重新设置选区
          currentPage.addParagraph();
          this.initSelection();
          return ;
      }
  
      const $last = $children.last();
      this.selection.createRangeByElem($last.children(0), false, true);
      if (this.selection.getRange()) {
        this.selection.restoreSelection();
      }
      this.selection.saveRange();
    }
  },
  // 绑定事件
  _bindEvent: function () {
  },
  // 解绑所有事件（暂时不对外开放）
  _offAllEvent: function () {
    $.offAll();
  }
}
export default yzWebeditor;
